import 'package:flutter/material.dart';
import 'package:adeptutils/adeptutils.dart';

class AdeptDropDownButtonFormField<T> extends StatefulWidget {

  AdeptValueController controller;
  AdeptValueController<List<DropdownMenuItem>> itemsController;
  String label;
  bool useTextUnspecified;
  String textUnspecified;
  List<String> entries;
  List<T> entryValues;

  Key key;
  T value;
  Widget hint;
  ValueChanged<T> onChanged;
  InputDecoration decoration;
  FormFieldSetter<T> onSaved;
  FormFieldValidator<T> validator;
  bool enabled;
  TextStyle textStyle;

  AdeptDropDownButtonFormField({
    this.controller,
    this.itemsController,
    this.label,
    this.useTextUnspecified = false,
    this.textUnspecified,
    this.entries,
    this.entryValues,
    this.key,
    this.value,
    this.hint,
    this.onChanged,
    this.decoration,
    this.onSaved,
    this.validator,
    this.enabled = true,
    this.textStyle,
  }) : super(
  ) {
    if(entries == null && entryValues == null){
      entries = [];
      entryValues = [];
    }
    assert(entries != null);
    assert(entryValues != null);
    assert(entries.length == entryValues.length);
    assert(enabled != null);
    decoration ??= InputDecoration();
    textUnspecified ??= "Unspecified";
    label ??= "";
    if(useTextUnspecified ??= false){
      entries.insert(0, textUnspecified);
      entryValues.insert(0, null);
    }
    controller ??= AdeptValueController();
    itemsController ??= AdeptValueController<List<DropdownMenuItem>>();
  }

  @override
  State createState()  => _AdeptDropDownButtonFormFieldState<T>();
}

class _AdeptDropDownButtonFormFieldState<T> extends State<AdeptDropDownButtonFormField>{
  Color colorPrimary, colorTextTertiary;
  List<DropdownMenuItem<T>> items;

  @override
  void initState() {
    super.initState();
    init();
    listeners();
  }

  void init() {
    generateFilterItems();
  }

  void generateFilterItems(){
    if(widget.itemsController.value == null || widget.itemsController.value.isEmpty){
      widget.itemsController.value = [];
      for(int i = 0; i < widget.entries.length; ++i){
        widget.itemsController.value.add(
            DropdownMenuItem<T>(
              value: widget.entryValues[i],
              child: Text(widget.entries[i], style: widget.textStyle,),
            )
        );
      }
    }
    items = widget.itemsController.value;
  }

  void listeners() {
    widget.controller.addListener(onValueChanged);
    widget.itemsController.addListener(onItemsChanged);
  }

  void onValueChanged(){
    if(widget.onChanged != null)
      widget.onChanged(widget.controller.value);
  }

  void onItemsChanged(){
    items = widget.itemsController.value ?? [];
    setState(() {});
  }

  void checkColors(context){
    colorPrimary ??= Theme.of(context).primaryColor;
    colorTextTertiary ??= Colors.grey.shade600;
  }

  @override
  Widget build(BuildContext context) {
    checkColors(context);
    return IgnorePointer(
      ignoring: !widget.enabled,
      child: Container(
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: <Widget>[
            !widget.useTextUnspecified ? Container() :
            Text(
              widget.label,
              style: (widget.textStyle ?? TextStyle()).copyWith(
                  color: widget.controller.value != null ? colorPrimary : colorTextTertiary, fontSize: 12),
            ),
            Divider(color: Colors.transparent, height: 4,),
            DropdownButtonFormField<T>(
              key: widget.key,
              hint: widget.hint,
              decoration: widget.decoration.copyWith(
                labelText: widget.useTextUnspecified ? null : widget.label,
                errorText: widget.controller.error,
                suffix: Container(width: 10,),
              ),
              validator: !widget.enabled ? null : widget.validator,
              onSaved: !widget.enabled ? null : widget.onSaved,
              items: items,
              value: widget.controller.value,
              onChanged: !widget.enabled ? null : (value){
                widget.controller.value = value;
                setState(() {});
              },
            ),
          ],
        ),
      ),
    );
  }

  @override
  void dispose() {
    widget.controller.removeListener(onValueChanged);
    widget.itemsController.removeListener(onItemsChanged);
    super.dispose();
  }
}