import 'package:flutter/material.dart';
import 'package:adeptutils/adeptutils.dart';

class TextFieldPreference extends StatefulWidget{
  final String title;
  final String summary;
  final String summaryPrefix;
  final String summarySuffix;
  final TextInputType textInputType;
  final String dialogMessage;
  final String localKey;
  final String defaultValue;
  final int maxLength;
  final String labelText;
  final String hintText;
  final Widget leading;
  final Widget trailing;
  final bool enabled;
  final bool useValueAsSummary;
  final bool useValueAsSummaryPrefix;
  final bool useValueAsSummarySuffix;
  final bool barrierDismissible;
  final bool resetOnException;
  final String acceptText;
  final String cancelText;
  final Function onEnable;
  final Function onDisable;
  final Function onChange;
  final Function(AdeptTextEditingController controller) onValidate;
  final Function(String value) onSave;

  TextFieldPreference(this.title, this.localKey,
      {
        this.summary,
        this.summaryPrefix,
        this.summarySuffix,
        this.dialogMessage,
        this.textInputType = TextInputType.text,
        this.defaultValue,
        this.maxLength,
        this.labelText,
        this.hintText,
        this.leading,
        this.trailing,
        this.enabled = true,
        this.useValueAsSummary = false,
        this.useValueAsSummaryPrefix = false,
        this.useValueAsSummarySuffix = false,
        this.barrierDismissible = true,
        this.resetOnException = true,
        this.acceptText = "ACCEPT",
        this.cancelText = "CANCEL",
        this.onEnable,
        this.onDisable,
        this.onChange,
        this.onValidate,
        this.onSave,
      });

  _TextFieldPreferenceState createState() => _TextFieldPreferenceState();
}

class _TextFieldPreferenceState extends State<TextFieldPreference> {
  String summary;
  String value;

  @override
  void initState() {
    super.initState();
    value = SharedPreferencesUtils.getString(widget.localKey, defaultValue: widget.defaultValue);
    updateSummary();
  }

  void updateSummary(){
    summary = widget.summary;
    if(summary == null) summary = "";
    summary =
        (widget.summaryPrefix != null ? widget.summaryPrefix : "") +
            (widget.useValueAsSummary && !TextUtils.isEmpty(value) ? value : widget.summary) +
            (widget.summarySuffix != null ? widget.summarySuffix : "");
    if(widget.useValueAsSummary && !TextUtils.isEmpty(value)) {
      summary = (widget.summaryPrefix != null ? widget.summaryPrefix : "") +
          value +
          (widget.summarySuffix != null ? widget.summarySuffix : "");
    }else
    if(widget.useValueAsSummaryPrefix && !TextUtils.isEmpty(value)) summary = "(${widget.summaryPrefix != null ? widget.summaryPrefix : ""}$value) $summary"; else
    if(widget.useValueAsSummarySuffix && !TextUtils.isEmpty(value)) summary = "$summary (${widget.summarySuffix != null ? widget.summarySuffix : ""}$value)";
  }

  @override
  Widget build(BuildContext context) {
    return ListTile(
        title: Text(widget.title),
        subtitle: summary != null ? Text(summary) : null,
        leading: widget.leading,
        trailing: widget.trailing,
        enabled: widget.enabled,
        onTap: () {
          showDialog(
            context: context,
            barrierDismissible: widget.barrierDismissible,
            builder: (context) => _CustomDialog(this),
          );
        },
      );
  }
}

class _CustomDialog extends StatefulWidget {
  final _TextFieldPreferenceState preference;

  _CustomDialog(this.preference);

  @override
  _CustomDialogState createState() => new _CustomDialogState();
}

class _CustomDialogState extends State<_CustomDialog> {
  final controller = AdeptTextEditingController();
  String error;

  @override
  void initState() {
    super.initState();
    controller.text = widget.preference.value;
  }

  @override
  Widget build(BuildContext context) {
    return AlertDialog(
      title: Text(widget.preference.widget.title),
      content: SingleChildScrollView(
        child: Column(
          children: <Widget>[
            !TextUtils.isEmpty(widget.preference.widget.dialogMessage) ?
            Text(widget.preference.widget.dialogMessage, style: TextStyle(color: Theme.of(context).textTheme.headline3.color)):
            Divider(color: Colors.transparent, height: 0,),
            Divider(color: Colors.transparent, height: !TextUtils.isEmpty(widget.preference.widget.dialogMessage) ? 16 : 0,),
            TextField(
              keyboardType: widget.preference.widget.textInputType ?? TextInputType.text,
              maxLength: widget.preference.widget.maxLength,
              controller: controller,
              decoration: InputDecoration(
                labelText: widget.preference.widget.labelText,
                hintText: widget.preference.widget.hintText,
                errorText: controller.error,
              ),
            ),
          ],
        ),
      ),
      actions: <Widget>[
        FlatButton(
          child: Text(widget.preference.widget.cancelText),
          onPressed: () {
            controller.text = widget.preference.value;
            controller.clearError();
            setState(() {});
            Navigator.of(context).pop();
          },
        ),
        FlatButton(
          child: Text(widget.preference.widget.acceptText),
          onPressed: () {
            try{
              controller.clearError();
              if(widget.preference.widget.onValidate != null) widget.preference.widget.onValidate(controller);
              SharedPreferencesUtils.putString(widget.preference.widget.localKey, widget.preference.value = controller.text);
              widget.preference.updateSummary();
              widget.preference.setState(() {});
              Navigator.of(context).pop();
              if(widget.preference.widget.onSave != null) widget.preference.widget.onSave(controller.text);
            } catch(e){
              controller.error = ExceptionUtils.parseException(e).toString();
              setState(() {});
            }
          },
        ),
      ],
    );
  }
}